@@ -48,13 +48,23 @@ Q_GLOBAL_STATIC(ReqSizeCalc, requestSizeCalculators);
 using ResSizeCalc = QHash<quint8, QModbusResponse::CalcFuncPtr>;
 Q_GLOBAL_STATIC(ResSizeCalc, responseSizeCalculators);
 
-namespace Private {
+struct QModbusPduPrivate
+{
+    QModbusPduPrivate() = delete;
+    Q_DISABLE_COPY_MOVE(QModbusPduPrivate)
 
 enum struct Type {
     Request,
     Response
 };
 
+/*!
+    \internal
+
+    Returns the minimum data size in bytes for the given \a pdu and the
+    Modbus PDU \a type. If the PDU's function code is invalid, undefined
+    or unknown, the return value will be \c {-1}.
+*/
 static int minimumDataSize(const QModbusPdu &pdu, Type type)
 {
     if (pdu.isException())
@@ -103,44 +113,53 @@ static int minimumDataSize(const QModbusPdu &pdu, Type type)
     return -1;
 }
 
-static QDataStream &pduFromStream(QDataStream &stream, QModbusPdu &pdu, Type type)
-{
-    quint8 codeByte = 0;
-    if (stream.readRawData((char *) (&codeByte), sizeof(quint8)) != sizeof(quint8))
-        return stream;
-    QModbusPdu::FunctionCode code = (QModbusPdu::FunctionCode) codeByte;
-    pdu.setFunctionCode(code);
+/*!
+    \internal
 
-    auto needsAdditionalRead = [](QModbusPdu &pdu, int size) -> bool {
-        if (size < 0)
-            pdu.setFunctionCode(QModbusResponse::Invalid);
-        if (size <= 0)
-            return false;
-        return true;
-    };
+    Extracts a Modbus PDU from a \a stream into the given \a pdu based on \a type.
+*/
+static QDataStream &pduFromStream(QDataStream &stream, Type type, QModbusPdu *pdu)
+{
+    struct RAII {
+        RAII(QModbusPdu *ptr = nullptr)
+            : tmp(ptr) {}
+        QModbusPdu *tmp{ nullptr };
+        ~RAII() { if (tmp) *tmp = {}; }
+    } raii = { pdu };
+
+    QModbusPdu::FunctionCode code = QModbusPdu::FunctionCode::Invalid;
+    if (stream.readRawData(reinterpret_cast<char *>(&code), sizeof(quint8)) != sizeof(quint8))
+        return stream;
+    pdu->setFunctionCode(code);
+
+    if (code == QModbusPdu::Invalid || code == QModbusPdu::UndefinedFunctionCode) // shortcut
+        return stream;
+
+    constexpr const int MaxPduDataSize = 252; // in bytes
+
+    // The calculateDataSize(...) function might need some data inside the
+    // given PDU argument to be able to figure out the right data size (e.g.
+    // WriteMultipleCoils contains some kind of "header"). So fake fill the PDU
+    // with the maximum available data but no more than the allowed max PDU
+    // data size.
+    QByteArray data(MaxPduDataSize, Qt::Uninitialized);
+    int read = stream.device()->peek(data.data(), MaxPduDataSize);
+    if (read < 0)
+        return stream;
+
+    data.resize(read);
+    pdu->setData(data);
 
     const bool isResponse = (type == Type::Response);
-    int size = isResponse ? QModbusResponse::minimumDataSize(pdu)
-                          : QModbusRequest::minimumDataSize(pdu);
-    if (!needsAdditionalRead(pdu, size))
-        return stream;
-
-    QByteArray data(size, Qt::Uninitialized);
-    if (stream.device()->peek(data.data(), data.size()) != size)
-        return stream;
-
-    pdu.setData(data);
-    size = isResponse ? QModbusResponse::calculateDataSize(pdu)
-                      : QModbusRequest::calculateDataSize(pdu);
-    if (!needsAdditionalRead(pdu, size))
-        return stream;
+    int size = isResponse ? QModbusResponse::calculateDataSize(*pdu)
+                          : QModbusRequest::calculateDataSize(*pdu);
 
     if (isResponse && (code == QModbusPdu::EncapsulatedInterfaceTransport)) {
         quint8 meiType;
-        pdu.decodeData(&meiType);
+        pdu->decodeData(&meiType);
         if (meiType == EncapsulatedInterfaceTransport::ReadDeviceIdentification) {
             int left = size, offset = 0;
-            while ((left > 0) && (size <= 252)) { // The maximum PDU data size is 252 bytes.
+            while ((left > 0) && (size <= MaxPduDataSize)) {
                 data.resize(size);
                 const int read = stream.readRawData(data.data() + offset, size - offset);
                 if ((read < 0) || (read != (size - offset))) {
@@ -152,35 +171,31 @@ static QDataStream &pduFromStream(QDataStream &stream, QModbusPdu &pdu, Type typ
                 left = QModbusResponse::calculateDataSize(QModbusResponse(code, data)) - offset;
                 size += left;
             }
-            if ((stream.status() == QDataStream::Ok) && (size <= 252)) {
-                pdu.setData(data);
+            if ((stream.status() == QDataStream::Ok) && (size <= MaxPduDataSize)) {
+                raii = {};
+                pdu->setData(data);
                 return stream; // early return to avoid second read
             }
         } else {
-            data.resize(stream.device()->size() - 1); // One byte for the function code.
+            data.resize(int(stream.device()->size() - 1)); // One byte for the function code.
         }
-    } else if (pdu.functionCode() == QModbusPdu::Diagnostics) {
+    } else if (pdu->functionCode() == QModbusPdu::Diagnostics) {
         quint16 subCode;
-        pdu.decodeData(&subCode);
+        pdu->decodeData(&subCode);
         if (subCode == Diagnostics::ReturnQueryData)
-            data.resize(stream.device()->size() - 1); // One byte for the function code.
+            data.resize(int(stream.device()->size() - 1)); // One byte for the function code.
     }
 
-    // reset what we have so far, next read might fail as well
-    pdu.setData(QByteArray());
-    pdu.setFunctionCode(QModbusPdu::Invalid);
-
-    if (data.size() <= 252) { // The maximum PDU data size is 252 bytes.
+    if (data.size() <= MaxPduDataSize) {
         data.resize(size);
         if (stream.readRawData(data.data(), data.size()) == size) {
-            pdu.setData(data);
-            pdu.setFunctionCode(code);
+            raii = {};
+            pdu->setData(data);
         }
     }
     return stream;
 }
-
-}   // namespace Private
+};
 
 /*!
     \class QModbusPdu
@@ -320,7 +335,7 @@ static QDataStream &pduFromStream(QDataStream &stream, QModbusPdu &pdu, Type typ
 
     \note Usage is limited \c quint8 and \c quint16 only. This is because
     \c QDataStream stream operators will not only append raw data, but also
-    e.g. size, count etc. for complex types.
+    e.g. size, count, etc. for complex types.
 */
 
 /*!
@@ -412,7 +427,7 @@ static QDataStream &pduFromStream(QDataStream &stream, QModbusPdu &pdu, Type typ
 
     \note Usage is limited \c quint8 and \c quint16 only. This is because
     \c QDataStream stream operators will not only append raw data, but also
-    e.g. size, count etc. for complex types.
+    e.g. size, count, etc. for complex types.
 */
 
 /*!
@@ -428,7 +443,7 @@ static QDataStream &pduFromStream(QDataStream &stream, QModbusPdu &pdu, Type typ
 
     \note Usage is limited \c quint8 and \c quint16 only. This is because
     \c QDataStream stream operators will not only append raw data, but also
-    e.g. size, count etc. for complex types.
+    e.g. size, count, etc. for complex types.
 */
 
 /*!
@@ -439,7 +454,7 @@ static QDataStream &pduFromStream(QDataStream &stream, QModbusPdu &pdu, Type typ
 QDebug operator<<(QDebug debug, const QModbusPdu &pdu)
 {
     QDebugStateSaver _(debug);
-    debug.nospace().noquote() << "0x" << hex << qSetFieldWidth(2) << qSetPadChar('0')
+    debug.nospace().noquote() << "0x" << Qt::hex << qSetFieldWidth(2) << qSetPadChar('0')
         << (pdu.isException() ? pdu.functionCode() | QModbusPdu::ExceptionByte : pdu.functionCode())
         << qSetFieldWidth(0) << pdu.data().toHex();
     return debug;
@@ -481,7 +496,7 @@ QDataStream &operator<<(QDataStream &stream, const QModbusPdu &pdu)
     \note When using the constructor taking the \c QByteArray, please make sure to convert the
         containing data to big-endian byte order before creating the request.
 
-    The same request can be created like this, if the values are know at compile time:
+    The same request can be created like this, if the values are known at compile time:
     \code
         quint16 startAddress = 19, numberOfCoils = 10;
         quint8 payloadInBytes = 2, outputHigh = 0xcd, outputLow = 0x01;
@@ -517,7 +532,7 @@ QDataStream &operator<<(QDataStream &stream, const QModbusPdu &pdu)
 
     \note Usage is limited \c quint8 and \c quint16 only. This is because
     \c QDataStream stream operators will not only append raw data, but also
-    e.g. size, count etc. for complex types.
+    e.g. size, count, etc. for complex types.
 */
 
 /*!
@@ -533,7 +548,7 @@ QDataStream &operator<<(QDataStream &stream, const QModbusPdu &pdu)
 */
 int QModbusRequest::minimumDataSize(const QModbusRequest &request)
 {
-    return Private::minimumDataSize(request, Private::Type::Request);
+    return QModbusPduPrivate::minimumDataSize(request, QModbusPduPrivate::Type::Request);
 }
 
 /*!
@@ -547,7 +562,7 @@ int QModbusRequest::minimumDataSize(const QModbusRequest &request)
 int QModbusRequest::calculateDataSize(const QModbusRequest &request)
 {
     if (requestSizeCalculators.exists()) {
-        if (auto ptr = requestSizeCalculators()->value(request.functionCode(), nullptr))
+        if (auto ptr = requestSizeCalculators()->value(quint8(request.functionCode()), nullptr))
             return ptr(request);
     }
 
@@ -555,7 +570,7 @@ int QModbusRequest::calculateDataSize(const QModbusRequest &request)
         return 1;
 
     int size = -1;
-    int minimum = Private::minimumDataSize(request, Private::Type::Request);
+    int minimum = QModbusPduPrivate::minimumDataSize(request, QModbusPduPrivate::Type::Request);
     if (minimum < 0)
         return size;
 
@@ -604,7 +619,7 @@ int QModbusRequest::calculateDataSize(const QModbusRequest &request)
 */
 void QModbusRequest::registerDataSizeCalculator(FunctionCode fc, CalcFuncPtr calculator)
 {
-    requestSizeCalculators()->insert(fc, calculator);
+    requestSizeCalculators()->insert(quint8(fc), calculator);
 }
 
 /*!
@@ -620,7 +635,7 @@ void QModbusRequest::registerDataSizeCalculator(FunctionCode fc, CalcFuncPtr cal
 */
 QDataStream &operator>>(QDataStream &stream, QModbusRequest &pdu)
 {
-    return Private::pduFromStream(stream, pdu, Private::Type::Request);
+    return QModbusPduPrivate::pduFromStream(stream, QModbusPduPrivate::Type::Request, &pdu);
 }
 
 /*!
@@ -638,7 +653,7 @@ QDataStream &operator>>(QDataStream &stream, QModbusRequest &pdu)
     \note When using the constructor taking the \c QByteArray, please make sure to convert the
         containing data to big-endian byte order before creating the request.
 
-    The same response can be created like this, if the values are know at compile time:
+    The same response can be created like this, if the values are known at compile time:
     \code
         quint8 payloadInBytes = 2, outputHigh = 0xcd, outputLow = 0x01;
         QModbusResponse response(QModbusResponse::ReadCoils, payloadInBytes, outputHigh, outputLow);
@@ -673,7 +688,7 @@ QDataStream &operator>>(QDataStream &stream, QModbusRequest &pdu)
 
     \note Usage is limited \c quint8 and \c quint16 only. This is because
     \c QDataStream stream operators will not only append raw data, but also
-    e.g. size, count etc. for complex types.
+    e.g. size, count, etc. for complex types.
 */
 
 /*!
@@ -689,7 +704,7 @@ QDataStream &operator>>(QDataStream &stream, QModbusRequest &pdu)
 */
 int QModbusResponse::minimumDataSize(const QModbusResponse &response)
 {
-    return Private::minimumDataSize(response, Private::Type::Response);
+    return QModbusPduPrivate::minimumDataSize(response, QModbusPduPrivate::Type::Response);
 }
 
 /*!
@@ -703,7 +718,7 @@ int QModbusResponse::minimumDataSize(const QModbusResponse &response)
 int QModbusResponse::calculateDataSize(const QModbusResponse &response)
 {
     if (responseSizeCalculators.exists()) {
-        if (auto ptr = responseSizeCalculators()->value(response.functionCode(), nullptr))
+        if (auto ptr = responseSizeCalculators()->value(quint8(response.functionCode()), nullptr))
             return ptr(response);
     }
 
@@ -711,7 +726,7 @@ int QModbusResponse::calculateDataSize(const QModbusResponse &response)
         return 1;
 
     int size = -1;
-    int minimum = Private::minimumDataSize(response, Private::Type::Response);
+    int minimum = QModbusPduPrivate::minimumDataSize(response, QModbusPduPrivate::Type::Response);
     if (minimum < 0)
         return size;
 
@@ -765,7 +780,7 @@ int QModbusResponse::calculateDataSize(const QModbusResponse &response)
         for (int i = 1; i < numOfObjects; ++i) {
             if (data.size() <= nextSizeField)
                 break;
-            objectSize = data[nextSizeField];
+            objectSize = quint8(data[nextSizeField]);
             size += objectSize;
             nextSizeField += objectSize + 2; // object size + object id field + object size field
         }
@@ -790,7 +805,7 @@ int QModbusResponse::calculateDataSize(const QModbusResponse &response)
 */
 void QModbusResponse::registerDataSizeCalculator(FunctionCode fc, CalcFuncPtr calculator)
 {
-    responseSizeCalculators()->insert(fc, calculator);
+    responseSizeCalculators()->insert(quint8(fc), calculator);
 }
 
 /*!
@@ -806,7 +821,7 @@ void QModbusResponse::registerDataSizeCalculator(FunctionCode fc, CalcFuncPtr ca
 */
 QDataStream &operator>>(QDataStream &stream, QModbusResponse &pdu)
 {
-    return Private::pduFromStream(stream, pdu, Private::Type::Response);
+    return QModbusPduPrivate::pduFromStream(stream, QModbusPduPrivate::Type::Response, &pdu);
 }
 
 /*!
